## @file
# split mosaic ODF
#
#
##



use strict;
use SAS;
use SAS::param;
use File::Path;
use File::Find;
use File::Copy;

sub splitmosaicodf () {
	
	## Get obsid parameter
	my $obsid=stringParameter("obsid");
	my $master_mosaic_odf = stringParameter("mosaicodfpath");
	$ENV{'SAS_ODF'} = $master_mosaic_odf;
	SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"Splitting ODF: $obsid");
	SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"mosaic ODF path: $master_mosaic_odf");
	
	##Set analysis directory
	my $Analysis_directory;
	chomp($Analysis_directory = `pwd`);
 	$Analysis_directory = $Analysis_directory."/";
	
	##Run odfingest
	my @args = ("odfingest");
    system(@args) == 0 or SAS::fatal("odfingest","error running odfingest");
    my @SAS_list  = glob('*SUM.SAS');
    print "@SAS_list\n";
    if ($#SAS_list == 0) {                    
		my $odfingestfile = $Analysis_directory.$SAS_list[0];
		SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"SUM.SAS file: $odfingestfile");		
		$ENV{'SAS_ODF'} =  $odfingestfile;
		SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"SAS_ODF variable set");		
    }     
    else
    {
    	SAS::fatal("SUM.SAS","file not found.");
    }
	##Run odfingest
	
	##Run preqgti
	my $preqgti_filename="preqgti_".$obsid.".fits";
	
	&run_preqgti($preqgti_filename);	
	##Run preqgti
	
	##Run pointing filter
	##Filter science raw data
	&run_pointingfilter($preqgti_filename);
	##Run pointing filter	
	
	#################################
	##Create sub ODFs directories
	##move the raw files to each one
	#################################
	
	##Get the number of pointings	
	chomp(my $pointings =`fkeyprint infile=$preqgti_filename+1  keynam=NAXIS2 outfil=STDOUT exact=yes | grep NAXIS2 | tail -1 | cut -c12-30`);
	
	SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"Number of stable poitings: $pointings");
	for (my $pointing=1;$pointing<=$pointings;$pointing++)
	{			
		my $new_pointing = 30 + $pointing;
		SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"Processing stable pointing number: $new_pointing");
		
		##Create the new obsid stable pointing var
		my $pseudo_obsid = substr($obsid, 0, 8).$new_pointing;
		
		
		my $sub_odf=$pseudo_obsid."/odf/";
		mkpath($sub_odf, 1, 0755);
		#mkdir (-p $sub_odf, 0755) or die "Cannot mkdir newdir: $!";
		SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"Copying ODF data");
		##Copy the master mosaic odf to the current sub-odf
		my $odffiles = $master_mosaic_odf."/*";
		my @ODF_files = glob($odffiles);
		foreach (@ODF_files ){
			print "Copying $_ to $sub_odf\n";
			SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"Copying $_ to $sub_odf");
			copy($_,$sub_odf."/");
		}		
		

		
		SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"New obsid: $pseudo_obsid");
		##move the filtered science files to sub odfs
		my $pfiles = "*".$pseudo_obsid."*.FIT";
		
		my @files = glob($pfiles);	
		SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"Moving science files");
		foreach (@files ){
			my $new_filename = $_;
			substr($new_filename, 13, 2) = "01";
			SAS::message($SAS::AppMsg,$SAS::NoisyMsg,"Old name $_ New name $new_filename");					
	    	move ($_, $sub_odf."/".$new_filename) or die $!;    		
		}
		
		##run odfrename for each stable pointing
		$ENV{'SAS_ODF'} =  $sub_odf;
		&run_odfrename($pointing,$preqgti_filename);
		##run odfrename for each stable pointing
	}			

	unlink 	$SAS_list[0];
	##Create sub ODFs
}


sub run_preqgti()
{
	my $filename = $_[0];
	
	my @args = ("preqgti preqgtifile=$filename ");
 		
    system(@args)== 0 or SAS::fatal("preqgti", "Error executing task");	
	
}

sub run_pointingfilter()
{
	my $filename = $_[0];
	
	my @args = ("pointingfilter preqgtifile=$filename ");
 		
    system(@args)== 0 or SAS::fatal("pointingfilter", "Error executing task");	
	
}
sub run_odfrename()
{
	my $pointing = $_[0];
	my $filename = $_[1];
	my @args = ("odfrename pointing=$pointing preqgtifile=$filename ");
	system(@args)== 0 or SAS::fatal("odfrename", "Error executing task");		
	
}
1;
